function [] = illustrateNNC()
%
% illustrates the use of NNC
% note that this function will sample a different seed set each
% time you run it, so the plots will differ.

% load an example Graph: coordinates x, adjacency matrix W
load('exampleGraph');
n = size(x,2); % number os points


% illustration: draw the sample points and graph
minx = min(min(x));
maxx = max(max(x));

figure(1);
subplot(1,2,1);
plot(x(1,:), x(2,:), 'k.'); hold on;
axis([minx maxx minx maxx]); 
axis equal;
axis off;
hold off;
title('sample points');


% draw graph
subplot(1,2,2);
plot(x(1,:), x(2,:), 'k.'); hold on;
for i=1:n
    neigh=find(W(i,:));
    for j=1:length(neigh)
        plot([x(1,i), x(1,neigh(j))], [x(2,i), x(2,neigh(j))], 'k');
    end
end
hold off;
axis([minx maxx minx maxx]); 
axis equal;
axis off;
title('graph');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTITION

% draw seeds
ns = ceil(log2(n)); % number of seed points
R = commdist(W);   % distance of points for assignment

% draw ns seed points
a = rand(n, 1);
[sorted, inds] = sort(a);
inds = inds(1:ns);
seeds = x(:,inds);

% assign points to closest seed
% for Ncut, we do this by commute distance
% (Euclidean for kmeans)
Rc = R(:, inds);
fnn = zeros(n, ns); % seed assignment indicator matrix
[minv, minind] = min(Rc,[],2);
for i=1:n 
    fnn(i,minind(i)) = 1;
end

figure;
subplot(1,2,1);
% plot assignment and make sure that seeds are assigned to themselves
[sorted, sortind] = sort(seeds(1,:));
cols = {'r','g','b','k','c','m','y'};
for i=1:ns
    is = sortind(i);
    
    % make sure that seeds are assigned to themselves
    fnn(inds(is),:) = 0;
    fnn(inds(is), is) = 1;
    
    % plot cluster of seed is
    printind = find(fnn(:,is));
    plot(x(1,printind), x(2,printind),[cols{mod(i,length(cols))+1} '.']); hold on;
    plot(seeds(1,is), seeds(2,is), [cols{mod(i,length(cols))+1} '*'], 'MarkerSize',14);
end
hold off;
axis([minx maxx minx maxx]); axis equal;
axis off;
title('seed assignments');


% contracted graph: choose nearest neighbour for each point
Wcont = fnn'*W*fnn;
    
% cluster
[fcont, nccont] = bbNcut(Wcont);
f = fnn*fcont; % assignment of all points

% plot result
subplot(1,2,2)
inds = find(f==1);
plot(x(1,inds), x(2,inds), 'b.'); hold on;
inds = find(f==0);
plot(x(1,inds), x(2,inds), 'r.'); hold off;
axis([minx maxx minx maxx]); axis equal;
axis off;
title(sprintf('clustering - Ncut value: %1.3f', nccont));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% commute distance

function [R] = commdist(W)
%
% returns (n-length(samples) x length(samples) commute distance submatrix
% W: adjacency matrix

D = diag(sum(W));
L = D - W;
n = size(W,1);
opts.issym = 1;
opts.disp=0;
Linv = pinv(L);
R = repmat(diag(Linv),1,n) + repmat(diag(Linv)',n,1) - 2*Linv;
